@@ -97,7 +97,7 @@ def get_contract_ids(api_client: Union[ApiClient, HistoryApiClient],
 	delivery_areas = [] if not delivery_areas else delivery_areas
 
 	# if minutes are entered wrong, raise exception
-	for t in [time_from, time_till]:
+	for t in [i for i in [time_from, time_till] if i]:
 		if t.minute not in [0, 15, 30, 45]:
 			raise ValueError("Time period has to start and end with one of these values: 0, 15, 30, 45")
 
