@@ -1,12 +1,12 @@
 import json
 import logging
 import re
-import urllib.parse
 from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from datetime import datetime
 from pathlib import Path
 from typing import Union, Optional, TypeVar, Any
+from urllib.parse import quote
 
 import pandas as pd
 import plotly.graph_objects as go
@@ -25,6 +25,7 @@ pandas_DataFrame = TypeVar('pandas.core.frame.DataFrame')
 ApiClient = TypeVar('powerbot_client.api_client.ApiClient')
 HistoryApiClient = TypeVar('powerbot_backtesting.models.history_api_models.HistoryApiClient')
 
+
 class BaseExporter(BaseModel, ABC):
     """
     Base class to all of PowerBot's exporter classes.
@@ -68,10 +69,13 @@ class BaseExporter(BaseModel, ABC):
         Returns:
             dict{key: DataFrame}: Dictionary of DataFrames
         """
+        if not hasattr(self, 'delivery_area') and not delivery_area:
+            raise ValueError("Delivery area has to be given")
+
         return pb.get_ohlc_data(trade_data=trade_data,
                                 timesteps=timesteps,
                                 time_unit=time_unit,
-                                delivery_area=delivery_area,
+                                delivery_area=self.delivery_area if hasattr(self, 'delivery_area') and not delivery_area else delivery_area,
                                 api_client=self.client if hasattr(self, 'client') else None,
                                 use_cached_data=use_cached_data,
                                 caching=caching,
@@ -89,6 +93,7 @@ class BaseExporter(BaseModel, ABC):
                        use_cached_data: bool = True,
                        caching: bool = True,
                        as_json: bool = False) -> dict[str, pd.DataFrame]:
+
         """
         Converts contract history data into order books in the specified timesteps. If no API client is passed, the function
         will automatically assume that the data is production data.
@@ -112,8 +117,11 @@ class BaseExporter(BaseModel, ABC):
         Returns:
             dict{key: DataFrame}: Dictionary of DataFrames
         """
+        if not hasattr(self, 'delivery_area') and not delivery_area:
+            raise ValueError("Delivery area has to be given")
+
         return pb.get_orderbooks(contract_hist_data=contract_hist_data,
-                                 delivery_area=delivery_area,
+                                 delivery_area=self.delivery_area if hasattr(self, 'delivery_area') and not delivery_area else delivery_area,
                                  timesteps=timesteps,
                                  time_unit=time_unit,
                                  timestamp=timestamp,
@@ -125,17 +133,21 @@ class BaseExporter(BaseModel, ABC):
 
     @staticmethod
     @validate_arguments
-    def get_orders(contract_hist_data: dict[str, pandas_DataFrame]) -> dict[str, list[dict]]:
+    def get_orders(contract_hist_data: dict[str, pandas_DataFrame],
+                   append_all: bool = False) -> dict[str, pandas_DataFrame]:
         """
-        Extracts all order data from contract history as is, without performing any quality control.
+        Extracts all order data from contract history as is, without performing any quality control. If necessary, orders for all contracts can be
+        appended to a single dataframe.
 
         Args:
             contract_hist_data (dict): Dictionary of Dataframes containing Contract History Data
+            append_all (bool): True if one dataframe containing all orders should be returned
 
         Returns:
-            dict{key: [orders]}: Dictionary of Lists of Orders
+            dict{key: pd.DataFrame}: Dictionary of DataFrames
         """
-        return pb.get_orders(contract_hist_data=contract_hist_data)
+        return pb.get_orders(contract_hist_data=contract_hist_data,
+                             append_all=append_all)
 
     @staticmethod
     @validate_arguments
@@ -143,16 +155,14 @@ class BaseExporter(BaseModel, ABC):
                          desired_depth: float,
                          min_depth: float = None) -> dict[str, float]:
         """
-        This method can be used to calculate the weighted average price for a dictionary of dataframes (e.g. orders, trades)
-        at a desired depth. The output is a singular value for each dataframe.
-        This function does not load any data, therefore the already existing data object has to be passed as an argument.
+        This method can be used to calculate the weighted average price for a dictionary of dataframes (e.g. orders, trades) at a desired depth.
+        The output is a singular value for each dataframe. This function does not load any data, therefore the already existing data object has to
+        be passed as an argument.
 
         Args:
-            objects (dict[str, DataFrame or list[str, dict]): A dictionary of dataframes, each of which needs to have a
-            'quantity' and a 'price' field. Alternatively, a list of dictionaries can be passed as well.
+            objects (dict[str, DataFrame): A dictionary of dataframes, each of which needs to have a 'quantity' and a 'price' field.
             desired_depth (float): The depth (in MW) specifying how many of the objects should be taken into consideration.
-            min_depth (float): The required minimum depth (in percent of the desired depth). If this requirement is not met,
-            return value is 0.
+            min_depth (float): The required minimum depth (in percent of the desired depth). If this requirement is not met, return value is 0.
 
         Returns:
             dict[str, float]: The weighted average price for the desired depth for each key in the dictionary.
@@ -161,6 +171,31 @@ class BaseExporter(BaseModel, ABC):
                                    desired_depth=desired_depth,
                                    min_depth=min_depth)
 
+    @staticmethod
+    @validate_arguments
+    def calc_rolling_vwap(trades: dict[str, pandas_DataFrame],
+                          rolling_interval: int = 1,
+                          rolling_time_unit: str = "hour") -> dict[str, pandas_DataFrame]:
+        """
+        This method can be used to calculate the rolling weighted average price for trades.
+        Every item in the given dataframe will be assigned the specific value weighted average price of all previous items that were executed in
+        the given time window (counting from the execution time of the current item).
+
+        Args:
+            trades (dict[str, DataFrame): A dictionary of dataframes containing trades
+            rolling_interval (int): The interval time that should be considered when calculating an items specific VWAP
+            rolling_time_unit (str): The time unit that VWAP should be calculated for. Possible: hour, minute, second
+
+        Returns:
+            dict[str, pd.Dataframe]: Original dictionary of dataframes, extended by weighted average price for the set time interval for each item
+        """
+        if rolling_time_unit not in ["hour", "minute", "second"]:
+            raise ValueError("rolling_time_unit needs to be one of the following: hour, minute, second")
+
+        return pb.calc_rolling_vwap(trades=trades,
+                                    rolling_interval=rolling_interval,
+                                    rolling_time_unit=rolling_time_unit)
+
     @staticmethod
     @validate_arguments
     def plot_ohlc(ohlc_data: dict[str, pandas_DataFrame],
@@ -254,7 +289,7 @@ class ApiExporter(BaseExporter):
     host: str = Field(description="URL of the PowerBot instance to connect to",
                       example="https://backup.powerbot-trading.com/{COMPANY NAME}/{EXCHANGE}/v2/api")
 
-    client: ApiClient = Field(description="Placeholder value, do not overwrite", default=None)
+    client: ApiClient = Field(description="Placeholder value, do not overwrite", default_factory=type(None))
 
     def __init__(self, **data: Any):
         super().__init__(**data)
@@ -276,8 +311,8 @@ class ApiExporter(BaseExporter):
 
     @validate_arguments
     def get_contract_ids(self,
-                         time_from: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default=None),
-                         time_till: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default=None),
+                         time_from: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default_factory=type(None)),
+                         time_till: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default_factory=type(None)),
                          contract_ids: list[str] = None,
                          contract_time: str = "all",
                          products: list[str] = None,
@@ -367,7 +402,7 @@ class ApiExporter(BaseExporter):
     def get_public_trades_by_days(self,
                                   previous_days: int,
                                   delivery_area: str,
-                                  time_from: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default=None),
+                                  time_from: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default_factory=type(None)),
                                   contract_time: str = None,
                                   contract_id: str = None) -> dict[str, pd.DataFrame]:
         """
@@ -528,7 +563,7 @@ class ApiExporter(BaseExporter):
                         contract_time: str,
                         delivery_area: str,
                         trade_data: dict[str, pandas_DataFrame] = None,
-                        time_from: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default=None),
+                        time_from: datetime = Field(description="Datetime in format yyyy-mm-dd hh:mm:ss", default_factory=type(None)),
                         previous_days: int = 10,
                         contract_id: str = None,
                         index: str = "ID3") -> pd.DataFrame:
@@ -568,7 +603,7 @@ class HistoryExporter(BaseExporter):
     Exporter class for interaction with the PowerBot History API and the subsequently created local __pb_cache__.
 
     This class can/ should be used when:
-        - the requested data has is older than at least 2-3 days and has already been made available via History API
+        - the requested data is older than at least 2-3 days and has already been made available via History API
         - the requested data is already stored in the local __pb_cache__ and has been loaded via History API.
 
     ATTENTION: loading historic data from the History API will create a json file containing all contract information
@@ -578,7 +613,7 @@ class HistoryExporter(BaseExporter):
     exchange: str = Field(description="The exchange data should be loaded for")
     delivery_area: str = Field(description="EIC-code of the delivery area data should be loaded for")
 
-    client: HistoryApiClient = Field(description="Placeholder value, do not overwrite", default=None)
+    client: HistoryApiClient = Field(description="Placeholder value, do not overwrite", default_factory=type(None))
 
     def __init__(self, **data: Any):
         super().__init__(**data)
@@ -766,8 +801,8 @@ class SQLExporter(BaseExporter):
     database: str = Field(description="Database name")
     port: int = Field(description="Database port")
 
-    logger: None = Field(description="Placeholder value, do not overwrite", default=None)
-    engine: None = Field(description="Placeholder value, do not overwrite", default=None)
+    logger: None = Field(description="Placeholder value, do not overwrite", default_factory=type(None))
+    engine: None = Field(description="Placeholder value, do not overwrite", default_factory=type(None))
 
     SQL_ERRORS = (
         InterfaceError,
@@ -787,6 +822,12 @@ class SQLExporter(BaseExporter):
         # Initialize Connection
         self.engine = self.__create_sql_engine()
 
+    @validator("user", "password")
+    def validate_user_and_password(cls, value):
+        value = quote(value)
+
+        return value
+
     @validator("db_type")
     def validate_db_type(cls, value):
         allowed_db_types = ['mysql', 'mariadb', 'postgresql', 'oracle', 'mssql', 'amazon_redshift', 'apache_drill',
@@ -821,7 +862,7 @@ class SQLExporter(BaseExporter):
                        "teradata_vantage": ["teradatasqlalchemy"]}
 
         self.logger.info("Now installing the following necessary package(s):\n"
-                        f"{db_packages[self.db_type]}")
+                         f"{db_packages[self.db_type]}")
 
         import subprocess
         import sys
@@ -868,7 +909,7 @@ class SQLExporter(BaseExporter):
 
         except (NoSuchModuleError, ModuleNotFoundError):
             self.logger.info("You currently do not have all the necessary packages installed to access a database of"
-                            f" type {self.db_type}.")
+                             f" type {self.db_type}.")
             return self.__install_packages()
 
         except ProgrammingError:
@@ -1190,7 +1231,7 @@ class SQLExporter(BaseExporter):
                           for i in result]
             return own_trades
 
-        if result and as_dataframe:
+        if as_dataframe:
             return pd.DataFrame(result)
 
         return result
@@ -1268,7 +1309,7 @@ class SQLExporter(BaseExporter):
                                for i in result]
             return internal_trades
 
-        if result and as_dataframe:
+        if as_dataframe:
             return pd.DataFrame(result)
 
         return result
@@ -1331,7 +1372,7 @@ class SQLExporter(BaseExporter):
 
             return signals
 
-        if result and as_dataframe:
+        if as_dataframe:
             return pd.DataFrame(result)
 
         return result
