@@ -61,7 +61,7 @@ def _get_private_data(api_client: ApiClient,
 		params["delivery_area"] = delivery_area
 
 	while more_obj:
-		new_objs = func_mapping[data_type](offset=offset, **param_mapping[data_type])
+		new_objs = func_mapping[data_type](offset=offset, **params)
 		if len(new_objs):
 			coll += new_objs
 			offset += len(new_objs)
@@ -470,7 +470,7 @@ def _historic_data_transformation(files: list, exchange: str, filetype: str) ->
 
 			df.quantity = df.quantity / 100
 			df.price = df.price / 100
-			df.prc_x_qty = df.prc_x_qty / 100
+			df.prc_x_qty = df.prc_x_qty / 10000
 
 		if filetype == "orders":
 			df.rename(columns={"asOf": "as_of", "bestAskPrice": "best_ask", "bestAskQuantity": "best_ask_qty",
@@ -483,7 +483,7 @@ def _historic_data_transformation(files: list, exchange: str, filetype: str) ->
 			# Getting information from details field if missing on upper level
 			meta_cols = {"best_bid": "bestBidPx", "best_ask": "bestAskPx", "best_bid_qty": "bestBidQty",
 						 "last_price": "lastPx", "last_qty": "lastQty", "last_trade_time": "lastTradeTime",
-						 "volume": "totalQty", "high": "highPx", "low": "lowPx"}
+						 "volume": "totalQty", "high": "highPx", "low": "lowPx", "revision_no": "revisionNo"}
 			missing_details = {k: v for k, v in meta_cols.items() if k not in df.columns}
 			details = df.details.tolist()
 
